@@ -25,17 +25,30 @@
 import httplib2
 import json
 import logging
-from urllib import quote
+
+# Python 3 changed the submodule for quote
+try:
+    from urllib import quote
+except ImportError:
+    from urllib.parse import quote
 
 
 class ChronosAPIError(Exception):
     pass
 
 
+class UnauthorizedError(Exception):
+    pass
+
+
 class MissingFieldError(Exception):
     pass
 
 
+class OneOfViolationError(Exception):
+    pass
+
+
 class ChronosClient(object):
     _user = None
     _password = None
@@ -88,6 +101,9 @@ class ChronosClient(object):
         """ List stats for a job """
         return self._call('/scheduler/job/stat/%s' % name, "GET")
 
+    def scheduler_graph(self):
+        return self._call('/scheduler/graph/csv', 'GET')
+
     def scheduler_stat_99th(self):
         return self._call('/scheduler/stats/99thPercentile', 'GET')
 
@@ -128,7 +144,7 @@ class ChronosClient(object):
                 response = self._check(*conn.request(endpoint, method, body=body, headers=hdrs))
                 self.logger.info('Got response from %s', endpoint)
                 return response
-            except Exception as e:
+            except ChronosAPIError as e:
                 self.logger.error('Error while calling %s: %s', endpoint, e.message)
 
         raise ChronosAPIError('No remaining Chronos servers to try')
@@ -137,11 +153,16 @@ class ChronosClient(object):
         status = resp.status
         self.logger.debug("status: %d" % status)
         payload = None
+
+        if status == 401:
+            raise UnauthorizedError()
+
         if content:
             try:
-                payload = json.loads(content)
+                payload = json.loads(content.decode('utf-8'))
             except ValueError:
-                self.logger.error("Response not valid json: %s" % content)
+                if resp['content-type'] == "application/json":
+                    self.logger.error("Response not valid json: %s" % content)
                 payload = content
 
         if payload is None and status != 204:
@@ -153,10 +174,13 @@ class ChronosClient(object):
         for k in ChronosJob.fields:
             if k not in job:
                 raise MissingFieldError("missing required field %s" % k)
-        for k in ChronosJob.one_of:
-            if k in job:
-                return True
-        raise MissingFieldError("Job must include one of %s" % ChronosJob.one_of)
+
+        if any(field in job for field in ChronosJob.one_of):
+            if len([field for field in ChronosJob.one_of if field in job]) > 1:
+                raise OneOfViolationError("Job must only include 1 of %s" % ChronosJob.one_of)
+        else:
+            raise MissingFieldError("Job must include one of %s" % ChronosJob.one_of)
+        return True
 
 
 class ChronosJob(object):
